@@ -8,14 +8,14 @@ import os
 import shutil
 import tempfile
 import uuid
-import importlib.resources as pkg_resources
+from importlib import resources
 from hashlib import md5
 from xml.sax.saxutils import quoteattr, escape
 from bs4 import BeautifulSoup
 from urllib.parse import quote
 
 from ybe.lib.utils import copy_ybe_resources
-from ybe.lib.ybe_contents import MultipleChoice, MultipleResponse, OpenQuestion, TextOnlyQuestion
+from ybe.lib.ybe_nodes import MultipleChoice, MultipleResponse, OpenQuestion, TextOnly
 
 
 def write_qti_zip(ybe_exam, fname, text_formatter=None):
@@ -149,14 +149,14 @@ def _write_assessment_meta(ybe_exam, dirname, assessment_identifier, text_format
         text_formatter (TextFormatter): specific actions to format the HTML text for use in the QTI.
     """
     template_items = {
-        'title': _escape_attr(ybe_exam.info.title),
-        'description': _escape_attr(text_formatter.format(ybe_exam.info.description or '')),
+        'title': _escape_attr(text_formatter.format(ybe_exam.info.title.to_html())),
+        'description': _escape_attr(text_formatter.format(ybe_exam.info.description.to_html())),
         'points_possible': float(ybe_exam.get_points_possible()),
         'assignment_identifier': uuid.uuid4().hex,
         'assessment_identifier': assessment_identifier,
         'assignment_group_identifier': uuid.uuid4().hex
     }
-    template = pkg_resources.read_text('ybe.data.qti_template', 'assessment_meta.xml')
+    template = resources.read_text('ybe.data.qti_template', 'assessment_meta.xml')
     assessment_meta = template.format(**template_items)
 
     with open(os.path.join(dirname, assessment_identifier, 'assessment_meta.xml'), 'w') as f:
@@ -173,11 +173,11 @@ def _write_questions_data(ybe_exam, dirname, assessment_identifier, text_formatt
         text_formatter (TextFormatter): specific actions to format the HTML text for use in the QTI.
     """
     template_items = {
-        'title': _escape_attr(ybe_exam.info.title),
+        'title': _escape_attr(text_formatter.format(ybe_exam.info.title.to_html())),
         'assessment_identifier': assessment_identifier,
         'questions': '\n'.join(_get_questions(ybe_exam, text_formatter)),
     }
-    template = pkg_resources.read_text('ybe.data.qti_template', 'assessment.xml')
+    template = resources.read_text('ybe.data.qti_template', 'assessment.xml')
     assessment = template.format(**template_items)
 
     with open(os.path.join(dirname, assessment_identifier, assessment_identifier + '.xml'), 'w') as f:
@@ -198,7 +198,7 @@ def _get_questions(ybe_exam, text_formatter):
         MultipleChoice: _convert_multiple_choice,
         MultipleResponse: _convert_multiple_response,
         OpenQuestion: _convert_open_question,
-        TextOnlyQuestion: _convert_text_only_question
+        TextOnly: _convert_text_only_question
     }
     return [converters[question.__class__](question, text_formatter) for question in ybe_exam.questions]
 
@@ -216,17 +216,13 @@ def _convert_multiple_choice(question, text_formatter):
     answer_ids = [uuid.uuid4().hex[0:6] for _ in question.answers]
     correct_answer_ind = [answer.correct for answer in question.answers].index(True)
 
+    feedback_items = _convert_feedback_items(question.feedback, text_formatter)
+
     answers = []
-    answer_template = '''
-        <response_label ident="{answer_id}">
-            <material>
-                <mattext texttype="text/html">{text}</mattext>
-            </material>
-        </response_label>
-    '''
     for answer_id, answer in zip(answer_ids, question.answers):
-        html = _escape_text(text_formatter.format(answer.text.to_html())).strip()
-        answers.append(answer_template.format(answer_id=answer_id, text=html).strip())
+        answers.append(_convert_multi_option_answer(answer_id, answer.text, text_formatter))
+        if answer.hint:
+            feedback_items.append(_convert_feedback(f'{answer_id}_fb', answer.hint, text_formatter))
 
     template_items = {
         'question_identifier': _escape_attr(question.id),
@@ -236,9 +232,11 @@ def _convert_multiple_choice(question, text_formatter):
         'question_text': _escape_text(text_formatter.format(question.text.to_html())),
         'original_answer_ids': ','.join(answer_ids),
         'correct_answer_id': answer_ids[correct_answer_ind],
-        'answers': '\n'.join(answers)
+        'answers': '\n'.join(answers),
+        'respconditions': '\n'.join(_create_multiple_choice_respconditions(question, answer_ids)),
+        'feedback_items': '\n'.join(feedback_items)
     }
-    template = pkg_resources.read_text('ybe.data.qti_template', 'assessment_multiple_choice.xml')
+    template = resources.read_text('ybe.data.qti_template', 'assessment_multiple_choice.xml')
     return template.format(**template_items).strip()
 
 
@@ -254,24 +252,13 @@ def _convert_multiple_response(question, text_formatter):
     """
     answer_ids = [uuid.uuid4().hex[0:6] for _ in question.answers]
 
-    answers = []
-    answer_template = '''
-        <response_label ident="{answer_id}">
-            <material>
-              <mattext texttype="text/html">{text}</mattext>
-            </material>
-          </response_label>
-    '''
-    for answer_id, answer in zip(answer_ids, question.answers):
-        html = _escape_text(text_formatter.format(answer.text.to_html())).strip()
-        answers.append(answer_template.format(answer_id=answer_id, text=html).strip())
+    feedback_items = _convert_feedback_items(question.feedback, text_formatter)
 
-    answer_conditions = []
+    answers = []
     for answer_id, answer in zip(answer_ids, question.answers):
-        if answer.correct:
-            answer_conditions.append(f'<varequal respident="response1">{answer_id}</varequal>')
-        else:
-            answer_conditions.append(f'''<not><varequal respident="response1">{answer_id}</varequal></not>''')
+        answers.append(_convert_multi_option_answer(answer_id, answer.text, text_formatter))
+        if answer.hint:
+            feedback_items.append(_convert_feedback(f'{answer_id}_fb', answer.hint, text_formatter))
 
     template_items = {
         'question_identifier': _escape_attr(question.id),
@@ -280,10 +267,11 @@ def _convert_multiple_response(question, text_formatter):
         'assessment_question_identifierref': uuid.uuid4().hex,
         'question_text': _escape_text(text_formatter.format(question.text.to_html())),
         'original_answer_ids': ','.join(answer_ids),
-        'answer_conditions': '\n'.join(answer_conditions),
-        'answers': '\n'.join(answers)
+        'answers': '\n'.join(answers),
+        'respconditions': '\n'.join(_create_multiple_response_respconditions(question, answer_ids)),
+        'feedback_items': '\n'.join(feedback_items)
     }
-    template = pkg_resources.read_text('ybe.data.qti_template', 'assessment_multiple_response.xml')
+    template = resources.read_text('ybe.data.qti_template', 'assessment_multiple_response.xml')
     return template.format(**template_items).strip()
 
 
@@ -297,14 +285,18 @@ def _convert_open_question(question, text_formatter):
     Returns:
         str: XML string with the question information
     """
+    feedback_items = _convert_feedback_items(question.feedback, text_formatter, allowed_items=['general'])
+
     template_items = {
         'question_identifier': _escape_attr(question.id),
         'question_title': 'Question',
         'points_possible': question.points,
         'assessment_question_identifierref': uuid.uuid4().hex,
-        'question_text': _escape_text(text_formatter.format(question.text.to_html()))
+        'question_text': _escape_text(text_formatter.format(question.text.to_html())),
+        'respconditions': '\n'.join(_create_open_question_respconditions(question)),
+        'feedback_items': '\n'.join(feedback_items)
     }
-    template = pkg_resources.read_text('ybe.data.qti_template', 'assessment_open_question.xml')
+    template = resources.read_text('ybe.data.qti_template', 'assessment_open_question.xml')
     return template.format(**template_items)
 
 
@@ -312,7 +304,7 @@ def _convert_text_only_question(question, text_formatter):
     """Convert an Ybe open question to a QTI XML string
 
     Args:
-        question (TextOnlyQuestion): text only question to convert
+        question (TextOnly): text only question to convert
         text_formatter (TextFormatter): specific actions to format the HTML text for use in the QTI.
 
     Returns:
@@ -324,7 +316,7 @@ def _convert_text_only_question(question, text_formatter):
         'assessment_question_identifierref': uuid.uuid4().hex,
         'question_text': _escape_text(text_formatter.format(question.text.to_html()))
     }
-    template = pkg_resources.read_text('ybe.data.qti_template', 'assessment_text_only_question.xml')
+    template = resources.read_text('ybe.data.qti_template', 'assessment_text_only_question.xml')
     return template.format(**template_items)
 
 
@@ -340,13 +332,13 @@ def _write_qti_manifest(ybe_exam, dirname, assessment_identifier, dependency_ide
     """
     template_items = {
         'manifest_identifier': uuid.uuid4().hex,
-        'title': _escape_attr(ybe_exam.info.title),
+        'title': _escape_attr(text_formatter.format(ybe_exam.info.title.to_html())),
         'date': ybe_exam.info.date.strftime('%Y-%m-%d'),
         'assessment_identifier': assessment_identifier,
         'dependency_identifier': dependency_identifier,
         'additional_resources': '\n'.join(_get_resources(ybe_exam, dirname)),
     }
-    template = pkg_resources.read_text('ybe.data.qti_template', 'imsmanifest.xml')
+    template = resources.read_text('ybe.data.qti_template', 'imsmanifest.xml')
     manifest_str = template.format(**template_items)
 
     with open(os.path.join(dirname, 'imsmanifest.xml'), 'w') as f:
@@ -380,6 +372,235 @@ def _get_resources(ybe_exam, dirname):
     return additional_resources
 
 
+def _create_open_question_respconditions(question):
+    """Create the response conditions for the open questions.
+
+    Args:
+        question (OpenQuestion): open question to create the response conditions for
+
+    Returns:
+        List[str]: the list with response conditions for this open question.
+    """
+    respconditions = []
+    if question.feedback.general is not None:
+        respconditions.append('''
+            <respcondition continue="Yes">
+                <conditionvar>
+                    <other/>
+                </conditionvar>
+                <displayfeedback feedbacktype="Response" linkrefid="general_fb"/>
+            </respcondition>
+        ''')
+    respconditions.append('''
+        <respcondition continue="No">
+            <conditionvar>
+                <other/>
+            </conditionvar>
+        </respcondition>
+    ''')
+    return respconditions
+
+
+def _create_multiple_choice_respconditions(question, answer_ids):
+    """Create the response conditions for the open questions.
+
+    Args:
+        question (MultipleChoice): question to create the response conditions for
+        answer_ids (List[str]): the list with the id's of the answers
+
+    Returns:
+        List[str]: the list with response conditions for this question.
+    """
+    correct_answer_id = answer_ids[[answer.correct for answer in question.answers].index(True)]
+
+    respconditions = []
+    if question.feedback.general:
+        respconditions.append('''
+            <respcondition continue="Yes">
+                <conditionvar>
+                    <other/>
+                </conditionvar>
+                <displayfeedback feedbacktype="Response" linkrefid="general_fb"/>
+            </respcondition>
+        ''')
+
+    for answer_id, answer in zip(answer_ids, question.answers):
+        if answer.hint:
+            respconditions.append(f'''
+                <respcondition continue="Yes">
+                    <conditionvar>
+                        <varequal respident="response1">{answer_id}</varequal>
+                    </conditionvar>
+                    <displayfeedback feedbacktype="Response" linkrefid="{answer_id}_fb"/>
+                </respcondition>
+            ''')
+
+    display_feedback = ''
+    if question.feedback.on_correct:
+        display_feedback = '<displayfeedback feedbacktype="Response" linkrefid="correct_fb"/>'
+
+    respconditions.append(f'''
+        <respcondition continue="No">
+            <conditionvar>
+                <varequal respident="response1">{correct_answer_id}</varequal>
+            </conditionvar>
+            <setvar action="Set" varname="SCORE">100</setvar>
+            {display_feedback}
+        </respcondition>
+    ''')
+
+    if question.feedback.on_incorrect:
+        respconditions.append(f'''
+            <respcondition continue="Yes">
+                <conditionvar>
+                    <other/>
+                </conditionvar>
+                <displayfeedback feedbacktype="Response" linkrefid="general_incorrect_fb"/>
+            </respcondition>
+        ''')
+
+    return respconditions
+
+
+def _create_multiple_response_respconditions(question, answer_ids):
+    """Create the response conditions for the open questions.
+
+    Args:
+        question (MultipleResponse): the multiple response question for which to generate the response conditions
+        answer_ids (List[str]): the list with the id's of the answers
+
+    Returns:
+        List[str]: the list with response conditions for this question.
+    """
+    answer_conditions = []
+    for answer_id, answer in zip(answer_ids, question.answers):
+        if answer.correct:
+            answer_conditions.append(f'<varequal respident="response1">{answer_id}</varequal>')
+        else:
+            answer_conditions.append(f'''<not><varequal respident="response1">{answer_id}</varequal></not>''')
+
+    respconditions = []
+
+    if question.feedback.general:
+        respconditions.append('''
+            <respcondition continue="Yes">
+                <conditionvar>
+                    <other/>
+                </conditionvar>
+                <displayfeedback feedbacktype="Response" linkrefid="general_fb"/>
+            </respcondition>
+        ''')
+
+    for answer_id, answer in zip(answer_ids, question.answers):
+        if answer.hint:
+            respconditions.append(f'''
+                <respcondition continue="Yes">
+                    <conditionvar>
+                        <varequal respident="response1">{answer_id}</varequal>
+                    </conditionvar>
+                    <displayfeedback feedbacktype="Response" linkrefid="{answer_id}_fb"/>
+                </respcondition>
+            ''')
+
+    display_feedback = ''
+    if question.feedback.on_correct:
+        display_feedback = '<displayfeedback feedbacktype="Response" linkrefid="correct_fb"/>'
+
+    respconditions.append(f'''
+        <respcondition continue="No">
+            <conditionvar>
+                <and>
+                    {answer_conditions}
+                </and>
+            </conditionvar>
+            <setvar action="Set" varname="SCORE">100</setvar>
+            {display_feedback}
+        </respcondition>
+    ''')
+
+    if question.feedback.on_incorrect:
+        respconditions.append(f'''
+            <respcondition continue="Yes">
+                <conditionvar>
+                    <other/>
+                </conditionvar>
+                <displayfeedback feedbacktype="Response" linkrefid="general_incorrect_fb"/>
+            </respcondition>
+        ''')
+
+    return respconditions
+
+
+def _convert_feedback_items(feedback, text_formatter, allowed_items=None):
+    """Convert the feedback items of a question.
+
+    Args:
+        feedback (Feedback): the feedback data
+        text_formatter (TextFormatter): specific actions to format the HTML text for use in the QTI.
+        allowed_items (list): the list of allowed feedback items
+
+    Returns:
+        List[str]: list with QTI elements for the feedback
+    """
+    allowed_items = allowed_items or ['general', 'on_correct', 'on_incorrect']
+
+    feedback_items = []
+    if feedback.general and 'general' in allowed_items:
+        feedback_items.append(_convert_feedback('general_fb', feedback.general, text_formatter))
+    if feedback.on_correct and 'on_correct' in allowed_items:
+        feedback_items.append(_convert_feedback('correct_fb', feedback.on_correct, text_formatter))
+    if feedback.on_incorrect and 'on_incorrect' in allowed_items:
+        feedback_items.append(_convert_feedback('general_incorrect_fb', feedback.on_incorrect, text_formatter))
+
+    return feedback_items
+
+
+def _convert_feedback(feedback_id, text, text_formatter):
+    """Convert the feedback items of the provided question into QTI format.
+
+    This will convert the general feedback
+
+    Args:
+        feedback_id (str): the id of the feedback
+        text (TextData): the data element containing the text to write
+        text_formatter (TextFormatter): specific actions to format the HTML text for use in the QTI.
+
+    Returns:
+        str: XML string with the question information
+    """
+    html = _escape_text(text_formatter.format(text.to_html())).strip()
+    return f'''
+        <itemfeedback ident="{feedback_id}">
+            <flow_mat>
+                <material>
+                    <mattext texttype="text/html">{html}</mattext>
+                </material>
+            </flow_mat>
+        </itemfeedback>
+    '''.strip()
+
+
+def _convert_multi_option_answer(answer_id, text, text_formatter):
+    """Convert a multiple choice or multiple response text to QTI format.
+
+    Args:
+        answer_id (str): the id of the answer
+        text (TextData): the data element containing the text to write.
+        text_formatter (TextFormatter): specific actions to format the HTML text for use in the QTI.
+
+    Returns:
+        str: the QTI formatted text
+    """
+    html = _escape_text(text_formatter.format(text.to_html())).strip()
+    return f'''
+        <response_label ident="{answer_id}">
+            <material>
+              <mattext texttype="text/html">{html}</mattext>
+            </material>
+          </response_label>
+    '''.strip()
+
+
 def _escape_attr(value):
     """Prepare the provided value for use as an attribute in xml
 
